<?php
/**
 * Copyright since 2019 SAS Makeup Bag
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to tech@202-ecommerce.com so we can send you a copy immediately.
 *
 * @author    202 ecommerce <tech@202-ecommerce.com>
 * @copyright Since 2019 SAS Makeup Bag
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License (AFL 3.0)
 */

namespace TrustbeautyreviewsAddon\Action;

use CartRule;
use Configuration;
use Currency;
use DateTime;
use Db;
use DbQuery;
use Language;
use Order;
use OrderState;
use Product;
use Tools;
use TrustbeautyReviews;

class GetOrdersAction implements ActionInterface
{
    public function handle($params = [])
    {
        $limit = isset($params['limit']) ? (int) $params['limit'] : 10000;

        $idShop = isset($params['id_shop']) ? (int) $params['id_shop'] : -1; // -1 : All shops

        $startDate = date('Y-m-d\TH:i:s', strtotime('-3 days'));
        $startDate = isset($params['start_date']) && $this->isValidDateTime($params['start_date'])
            ? $params['start_date'] : $startDate;

        $endDate = isset($params['end_date']) && $this->isValidDateTime($params['end_date'])
            ? $params['end_date'] : '';

        $confOrders = Configuration::get(TrustbeautyReviews::ORDERS_STATUS) !== false ?
            Configuration::get(TrustbeautyReviews::ORDERS_STATUS) :
            json_encode([Configuration::get('PS_OS_DELIVERED')]);

        $success = false;

        $count = 0;

        try {
            $dataOrders = $this->getOrders($idShop, json_decode($confOrders), $startDate, $endDate, $limit);
            $count = count($dataOrders);
            $success = true;
        } catch (\Exception $ex) {
            $dataOrders = ['error' => $ex];
        }

        return [
            'meta' => [
                'success' => $success,
                'count' => $count,
                'platform' => 'prestashop',
                'ps_version' => _PS_VERSION_,
                'module_version' => TrustbeautyReviews::VERSION,
                'version' => '1.0.0',
            ],
            'data' => $dataOrders,
        ];
    }

    /**
     * @param string date
     *
     * @return bool
     */
    protected function isValidDateTime($sDate)
    {
        return (DateTime::createFromFormat('Y-m-d\TH:i:s', $sDate)) !== false;
    }

    /**
     *  Get all order to esport
     *
     * @param int $idShop (-1 = all shops)
     * @param array $status (order status)
     * @param string $startDate
     * @param string $endDate
     * @param int $limit
     *
     * @return array $aLangs
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    private function getOrders($idShop, $status, $startDate, $endDate, $limit)
    {
        $query = new DbQuery();
        $query->select('id_order, date_upd')
            ->from('orders');

        if (!empty($idShop) && $idShop > 0) {
            $query->where('`id_shop` =' . (int) $idShop);
        }

        if (!empty($startDate)) {
            $startDateTime = DateTime::createFromFormat('Y-m-d\TH:i:s', $startDate);
            $query->where('`date_upd` >= \'' . $startDateTime->format('Y-m-d H:i:s') . '\'');
        }
        if (!empty($endDate)) {
            $endDateTime = DateTime::createFromFormat('Y-m-d\TH:i:s', $endDate);
            $query->where('`date_upd` <= \'' . $endDateTime->format('Y-m-d H:i:s') . '\'');
        }

        $query->orderBy('`date_upd` DESC');

        if ($limit > 0) {
            $query->limit($limit);
        }

        $orders = Db::getInstance()->executeS($query);

        // Order and Products Info
        $aOrders = [];
        $languages = $this->getLanguages($idShop);

        foreach ($orders as $aOrder) {
            $orderIn = new Order((int) $aOrder['id_order']);
            $currency = new Currency($orderIn->id_currency, $orderIn->id_lang, $orderIn->id_shop);
            $orderState = new OrderState($orderIn->getCurrentState(), (int) Configuration::get('PS_LANG_DEFAULT'));
            $dateAdd = new DateTime($orderIn->date_add);
            $dateUpd = new DateTime($orderIn->date_upd);
            $aOrders[] = [
                'id' => (int) $orderIn->id,
                'reference' => (string) $orderIn->reference,
                'date' => $dateAdd->format('Y-m-d\TH:i:sP'),
                'date_update' => $dateUpd->format('Y-m-d\TH:i:sP'),
                'id_shop' => (int) $orderIn->id_shop,
                'lang' => (string) isset($languages[$orderIn->id_lang]) ? $languages[$orderIn->id_lang] : '??',
                'customer' => $this->getCustomer($orderIn),
                'currency' => $currency->iso_code,
                'total_paid_tax_incl' => Tools::ps_round($orderIn->total_paid_tax_incl, 2),
                'total_paid_tax_excl' => Tools::ps_round($orderIn->total_paid_tax_excl, 2),
                'total_shipping_tax_incl' => Tools::ps_round($orderIn->total_shipping_tax_incl, 2),
                'coupons' => $this->getCartRules($orderIn),
                'state' => $orderState->name,
                'send_email' => (bool) in_array($orderIn->getCurrentState(), $status),
                'products' => (array) $this->getProductDetails($orderIn),
            ];
        }

        return $aOrders;
    }

    /**
     *  Get all shop languages
     *
     *  @param int $idShop (-1 = all shops)
     *
     *  @return array $arrayLanguages
     */
    protected function getLanguages($idShop)
    {
        $arrayLanguages = [];
        $languages = $idShop == -1 ? Language::getLanguages(true) : Language::getLanguages(true, $idShop);
        foreach ($languages as $aLanguage) {
            $idLang = (int) $aLanguage['id_lang'];
            $arrayLanguages[$idLang] = $aLanguage['iso_code'];
        }

        return $arrayLanguages;
    }

    protected function getCartRules(Order $order)
    {
        return array_map(function ($cartRule) {
            $cartRuleObject = new CartRule($cartRule['id_cart_rule']);

            return $cartRuleObject->code;
        }, $order->getCartRules());
    }

    /**
     *  Get customer details by order
     *
     * @param mixed $order
     *
     * @return array $customerIn
     *
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    protected function getCustomer(Order $order)
    {
        $customerObj = new \Customer($order->id_customer);
        $customerIn = [];
        if (\Validate::isLoadedObject($customerObj)) {
            $dateAdd = str_replace(' ', 'T', $customerObj->date_add);
            $customerIn = [
                'id' => $order->id_customer,
                'email' => $customerObj->email,
                'firstname' => $customerObj->firstname,
                'lastname' => $customerObj->lastname,
                'date_add' => $dateAdd,
            ];
        }

        return $customerIn;
    }

    /**
     *  Get product details by order
     *
     *  @param Order $order
     *
     *  @return array $productsIn
     */
    protected function getProductDetails(Order $order)
    {
        $products = $order->getProducts();
        $productsIn = [];
        $address = new \Address($order->id_address_invoice);
        foreach ($products as $inProduct) {
            $specificPriceOutput = null;
            $catalogPrice = Product::priceCalculation(
                $order->id_shop, // Shop id
                (int) $inProduct['id_product'], // Product id
                (int) $inProduct['product_attribute_id'], // Product attribute id
                $address->id_country, // Country id
                null, // State id
                null, // ZipCode
                $order->id_currency, // Currency Id
                null, // Group id
                (int) $inProduct['product_quantity'], // Quantity
                true, // use_tax with or without tax
                2, // Decimals (round)
                false, // only_redud
                false, // use_reduc
                false, // with ecotax
                $specificPriceOutput, // specific price
                false // use group reduction
            );
            $productsIn[] = [
                'id_product' => (int) $inProduct['id_product'],
                'id_attribute' => $inProduct['product_attribute_id'],
                'quantity' => (int) $inProduct['product_quantity'],
                'unit_price_incl' => Tools::ps_round($inProduct['unit_price_tax_incl'], 2),
                'unit_price_catalog_incl' => Tools::ps_round($catalogPrice, 2),
                'label' => (string) $inProduct['product_name'],
            ];
        }

        return $productsIn;
    }
}
